/*
================================================================
simv - simple GTK image viewer
usage:
    simv
        Opens as a GUI app

    simv filename [-p] [nx ny]
        Opens the specified image file in an UNDECORATED window.
        If specified, the window is opened at the coordinates (nx, ny)

        If the "-p" option is used, the window is created as a
        pop-up window: it will stay on top of other windows.
        
        If <filename> starts with "-" then the usage syntax is printed
        to stderr and the program exits.

simv is based on the following source code:        
https://developer.gnome.org/gnome-devel-demos/stable/image-viewer.c.html.en
================================================================
*/

/*
================================================================
To compile:

gcc simv.c -o simv `pkg-config --cflags --libs gtk+-2.0`
================================================================
*/

#include <gtk/gtk.h>


#include <glib/gi18n.h>

static void
on_open_image (GtkButton* button, gpointer user_data)
{
    GtkWidget *image = GTK_WIDGET (user_data);
    GtkWidget *toplevel = gtk_widget_get_toplevel (image);
    GtkFileFilter *filter = gtk_file_filter_new ();
    GtkWidget *dialog = gtk_file_chooser_dialog_new (_("Select image"),
                                                     GTK_WINDOW (toplevel),
                                                     GTK_FILE_CHOOSER_ACTION_OPEN,
                                                     GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
                                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                                     NULL);

    gtk_file_filter_add_pixbuf_formats (filter);
    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog),
                                 filter);
    
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);

    switch (gtk_dialog_run (GTK_DIALOG (dialog)))
    {
        case GTK_RESPONSE_ACCEPT:
        {
            gchar *filename = 
                gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
            gtk_image_set_from_file (GTK_IMAGE (image), filename);
            break;
        }
        default:
            break;
    }
    gtk_widget_destroy (dialog);
}
    

static GtkWidget*
create_window (void)
{
    GtkWidget *window;
    GtkWidget *button;
    GtkWidget *image;
    GtkWidget *box;

    /* Set up the UI */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), "simv");

    box = gtk_vbox_new (FALSE, 5 );
    button = gtk_button_new_with_label (_("Select image"));
    image = gtk_image_new ();

    gtk_box_pack_start (GTK_BOX (box), image, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);

    gtk_container_add (GTK_CONTAINER (window), box);

    /* Connect signals */

    /* Show open dialog when opening a file */
    g_signal_connect (button, "clicked", G_CALLBACK (on_open_image), image);
    
    /* Exit when the window is closed */
    g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);
    
    return window;
}

/*
======================================
    IMAGE ONLY VERSION!!!!
======================================
*/
    
static GtkWidget*
create_window_image_only (char* filename, int popup)
{
    GtkWidget *window;
    GtkWidget *image;
    GtkWidget *box;
    GtkWidget *eventBox;

    if (popup)
        window = gtk_window_new (GTK_WINDOW_POPUP);
    else
        window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    gtk_window_set_decorated (GTK_WINDOW (window), FALSE);

    eventBox = gtk_event_box_new ();
    
    image = gtk_image_new ();

    gtk_container_add (GTK_CONTAINER (window), eventBox);
    gtk_widget_add_events (eventBox, GDK_BUTTON_PRESS_MASK);

    gtk_image_set_from_file (GTK_IMAGE (image), filename);

    gtk_container_add (GTK_CONTAINER (eventBox), image);

    /* IMAGE ONLY VERSION!!! */

    g_signal_connect (eventBox, "button-press-event", G_CALLBACK (gtk_main_quit), NULL);
    g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);
    
    return window;
}


int
main (int argc, char *argv[])
{
    GtkWidget *window;

#ifdef ENABLE_NLS
    bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
    textdomain (GETTEXT_PACKAGE);
#endif
    
    gtk_init (&argc, &argv);

    if (argc == 1)
    {
        window = create_window ();
    }
    else if (argv[1][0] == '-')
    {
        fprintf (stderr, "usage: simv [filename [-p] [x-coord y-coord]]\n" );
        return 0;
    }
    else
    {
        int popup = 0;
        int xarg = 2;
        
        if (argc >= 3 && argv[2][0] == '-' && argv[2][1] == 'p')
        {
            popup = 1;
            xarg++;
        }
            
        window = create_window_image_only (argv[1], popup);

        if (argc >= 4)
        {
            int x = atoi (argv[xarg]);
            int y = atoi (argv[xarg+1]);

            x = (x<0) ? 0 : x;
            y = (y<0) ? 0 : y;

            gtk_window_move( GTK_WINDOW (window), x, y);
        }
    }

    gtk_widget_show_all (window);

    gtk_main ();
    return 0;
}
